package cn.wise.sc.consume.business.security.jwt;

import java.util.Calendar;

import cn.wise.sc.consume.business.model.LoginUser;
import cn.wise.sc.consume.business.security.JwtAuthenticationToken;
import org.springframework.security.authentication.AuthenticationProvider;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.web.authentication.www.NonceExpiredException;
import com.auth0.jwt.JWT;
import com.auth0.jwt.JWTVerifier;
import com.auth0.jwt.algorithms.Algorithm;
import com.auth0.jwt.interfaces.DecodedJWT;

public class JwtAuthenticationProvider implements AuthenticationProvider{
	
	private JwtUserService userService;

	public JwtAuthenticationProvider(JwtUserService userService) {
		this.userService = userService;
	}

	@Override
	public Authentication authenticate(Authentication authentication) throws AuthenticationException {
		DecodedJWT jwt = ((JwtAuthenticationToken)authentication).getToken();
		if(jwt.getExpiresAt().before(Calendar.getInstance().getTime())) {
			throw new NonceExpiredException("账号登陆时效已过请重新登陆");
		}
		String username = jwt.getSubject();
		LoginUser user = userService.getUserLoginInfo(username);

		/*if(!user.getTenantId().equals("0")){
			SysTenant tenant = tenantDao.getByTenantId(user.getTenantId());
			if(null == tenant || tenant.getIsDelete() == 0 ){
				throw new NonceExpiredException("租户信息已删除");
			}
			if(tenant.getStatus() == 0){
				throw new NonceExpiredException("企业申请已通过，请联系管理员审核");
			}
		}*/
		if(user == null || user.getPassword()==null) {
			throw new NonceExpiredException("账号登陆时效已过请重新登陆");
		}
		String encryptSalt = user.getPassword();
		try {
            Algorithm algorithm = Algorithm.HMAC256(encryptSalt);
            JWTVerifier verifier = JWT.require(algorithm)
                    .withSubject(username)
                    .build();
            verifier.verify(jwt.getToken());
        } catch (Exception e) {
            throw new BadCredentialsException("JWT token认证失败", e);
        }
		JwtAuthenticationToken token = new JwtAuthenticationToken(user, jwt, user.getAuthorities());
		return token;
	}

	@Override
	public boolean supports(Class<?> authentication) {
		return authentication.isAssignableFrom(JwtAuthenticationToken.class);
	}


}
