package cn.wise.sc.consume.business.security.jwt;
import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;
import cn.wise.sc.consume.business.entity.SysPermission;
import cn.wise.sc.consume.business.entity.SysUser;
import cn.wise.sc.consume.business.mapper.SysPermissionMapper;
import cn.wise.sc.consume.business.mapper.SysUserMapper;
import cn.wise.sc.consume.business.model.LoginUser;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.security.authentication.AuthenticationCredentialsNotFoundException;
import org.springframework.security.authentication.DisabledException;
import org.springframework.security.authentication.LockedException;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.bcrypt.BCrypt;

import com.auth0.jwt.JWT;
import com.auth0.jwt.algorithms.Algorithm;
import org.springframework.stereotype.Service;

@Service
public class JwtUserService implements UserDetailsService {
//    @Autowired
//    private PasswordEncoder passwordEncoder;
    @Autowired
    private RedisTemplate redisTemplate;
    @Autowired
    private SysUserMapper userMapper;
    @Autowired
    private SysPermissionMapper permissionDao;




  /*  public JwtUserService() {
        //默认使用 bcrypt， strength=10
        this.passwordEncoder = PasswordEncoderFactories.createDelegatingPasswordEncoder();
    }*/

    public LoginUser getUserLoginInfo(String username) {
        /**
         * 从数据库或者缓存中取出jwt token生成时用的salt
         */
        String salt = (String) redisTemplate.opsForValue().get("token:" + username);
        LoginUser user = (LoginUser) loadUserByUsername(username);
        //将salt放到password字段返回
		user.setPassword(salt);
        return user;
    }

    public String saveUserLoginInfo(UserDetails user) {
        String salt = BCrypt.gensalt();
        /**
         * 将salt保存到数据库或者缓存中
         */
        redisTemplate.opsForValue().set("token:" + user.getUsername(), salt, 3600, TimeUnit.SECONDS);
        Algorithm algorithm = Algorithm.HMAC256(salt);
        Date date = new Date(System.currentTimeMillis() + 3600 * 1000);  //设置1小时后过期
        return JWT.create()
                .withNotBefore(new Date(System.currentTimeMillis()))
                .withSubject(user.getUsername())
                .withExpiresAt(date)
                .withIssuedAt(new Date(System.currentTimeMillis()))
                .sign(algorithm);
    }

    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        SysUser sysUser = userMapper.getUser(username.split("&")[0]);
        if (sysUser == null) {
            throw new AuthenticationCredentialsNotFoundException("用户名不存在");
        } else if (sysUser.getStatus() == SysUser.Status.LOCKED) {
            throw new LockedException("用户被锁定,请联系管理员");
        } else if (sysUser.getStatus() == SysUser.Status.DISABLED) {
            throw new DisabledException("用户已作废");
        }
        LoginUser loginUser = new LoginUser();
        BeanUtils.copyProperties(sysUser, loginUser);


		if(username!=null && username.split("&").length==1){
			List<SysPermission> permissions = permissionDao.listByUserId(sysUser.getId());
			loginUser.setPermissions(permissions);
		}

        return loginUser;
    }


    public void deleteUserLoginInfo(String username) {
        /**
         * 清除数据库或者缓存中登录salt
         */
        redisTemplate.delete("token:" + username);

    }
}
